/******************************************************************************
 *
 *    File Name:  tb_top.v
 *      Version:  2.5
 *         Date:  June 05, 2000
 * 
 *  Description:  Functional test bench.
 * 
 *       Author:  Jennnifer Tran
 *      Company:  Xilinx
 *
 *                Copyright (c) 1999 Xilinx, Inc.
 *                All rights reserved
 *
 *    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
 * 
 *   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
 *                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
 *                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 *                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
 *
 ******************************************************************************/

`include "../src/define.v"
`include "../ram_model/mt46v4m16-133.v"
`include "../src/top.v"
`include "../src/user_int.v"
`include "../src/ddr_ctlr.v"
`include "../src/clk_dlls.v"
`include "../src/data_path.v"
`include "../src/addr_latch.v"
`include "../src/controller.v"
`include "../src/brst_cntr.v"
`include "../src/cslt_cntr.v"
`include "../src/rcd_cntr.v"
`include "../src/ddr_dq_io_16.v"

`timescale 1ns / 1ps 
`define PERIOD       10
`define HALF_PERIOD  `PERIOD/2      
`define DELAY_TIME        `PERIOD/10

module test();   
   reg [`SYS_ADDR_MSB:0] sys_addr;
   reg [7:1] 		 sys_cmd;
   reg 			 sys_reset_n;
   reg 			 sys_clk;
   reg [`SYS_DATA_MSB:0] sys_data_i;
   
   wire [`DDR_ADDR_MSB:0] ddr_ad;
   wire [`DDR_DATA_MSB:0] ddr_dq; 
   wire [1:0] 		  ddr_dm;
   wire [1:0] 		  ddr_ba;
   wire [1:0] 		  ddr_dqs;
   wire 		  ddr_rasb;
   wire 		  ddr_casb;
   wire 		  ddr_web;

   wire [`SYS_DATA_MSB:0] sys_data_o;
   
   reg 			  GSR;
   
`include "string_decode_fn.v"
   top utt(.ddr_ad(ddr_ad), .ddr_dm(ddr_dm), .ddr_ba(ddr_ba), 
	   .ddr_rasb(ddr_rasb), .ddr_casb(ddr_casb), .ddr_web(ddr_web), 
	   .ddr_clk(ddr_clk), .ddr_clkb(ddr_clkb), .ddr_dqs(ddr_dqs), 
	   .ddr_csb(ddr_csb), .ddr_cke(ddr_cke), 
	   .ddr_dq(ddr_dq), 
	   .sys_addr(sys_addr), .sys_data_i(sys_data_i), 
	   .sys_cmd(sys_cmd), .sys_reset_n(sys_reset_n), .sys_clk(sys_clk), 
	   .sys_clk_fb(sys_clk_fb), .sys_data_o(sys_data_o), .sys_ref_ack(sys_ref_ack),
	   .sys_data_valid(sys_data_valid)
	   );

   assign  sys_clk_fb = ddr_clk;  //feed DDR clock back to FPGA 
   
   mt46v4m16 u_ddr (.Clk(ddr_clk), .Clk_n(ddr_clkb), .Cs_n(ddr_csb), 
	      .Cke(ddr_cke), .Ba(ddr_ba), .Addr(ddr_ad), 
	      .Ras_n(ddr_rasb), .Cas_n(ddr_casb), .We_n(ddr_web), 
	      .Dm(ddr_dm), .Dq(ddr_dq), .Dqs(ddr_dqs[0]));
  
   always #(`HALF_PERIOD) sys_clk <= ~sys_clk;

   //at startup, all flip-flops would be reset by GSR
   //simulate GSR function
   initial begin
      force utt.I_ddr_ctlr.cslt_cntr.count = 3'b000;
      #1;
      release utt.I_ddr_ctlr.cslt_cntr.count;
   end
   
   initial begin
      sys_cmd = 7'b0000000;
      sys_addr = 22'h00000;
      sys_data_i = 32'h0000000;
      sys_clk = 0;
      sys_reset_n = 1;
      sys_cmd[`SYS_NOP] = 1;
     
      
      #(32*`PERIOD);   //wait for DLL in FPGA to lock
      #(`HALF_PERIOD); //delay inputs to avoid race condition
      #(`DELAY_TIME); precharge_all_banks;
      #(5*`PERIOD);     nop;
      #(`PERIOD);     EMRS;
      
      run_test(3'b011,3'b010); // burst = 8, cas latency = 2;
      run_test(3'b011,3'b110); // burst = 8, cas latency = 2.5;
		     
      run_test(3'b010,3'b010); // burst = 4, cas latency = 2;
      run_test(3'b010,3'b110); // burst = 4, cas latency = 2.5;
		     
      run_test(3'b001,3'b010); // burst = 2, cas latency = 2;
      run_test(3'b001,3'b110); // burst = 2, cas latency = 2.5;

      $finish;
   end // initial begin
   
   task run_test;
      input [2:0] burst_length;
      input [2:0] cas_latency;
      begin
	 #(`PERIOD);     nop;
	 #(`PERIOD);     MRS(burst_length,0,cas_latency); 
	 #(`PERIOD);     nop;
	 #(`PERIOD);     precharge_all_banks;
	 #(`PERIOD);     nop;
	 #(2*`PERIOD);   auto_refresh;
	 #(`PERIOD);     nop;
	 #(6*`PERIOD);   auto_refresh;
	 #(`PERIOD);     nop;
	 //write_autoprecharge (ba,row_addr,col_addr,write_data1,write_data2,write_data3,write_data4)
	 #(6*`PERIOD);   write_autoprecharge(2'b00, 12'h200, 8'h10, 32'h10102020, 32'h03030404, 32'h50506060, 32'h07070808);
	 #(`PERIOD);     nop;
	 #(10*`PERIOD);  read_autoprecharge(2'b00, 12'h200, 8'h10);
	 #(9*`PERIOD);   nop;
	 #(10*`PERIOD);  write_autoprecharge(2'b01, 12'h200, 8'h20, 32'h09876543, 32'h98765432, 32'h87654321, 32'h76543210);
	 #(`PERIOD);     nop;
	 #(10*`PERIOD);  read_autoprecharge(2'b00, 12'h200, 8'h10);
	 #(9*`PERIOD);   nop;
	 #(10*`PERIOD);  read_autoprecharge(2'b01, 12'h200, 8'h20);
	 #(9*`PERIOD);   nop;
	 #(10*`PERIOD);
      end
   endtask // run_test   

   task EMRS;
      begin
	 sys_cmd = `sys_load_mr;
	 sys_addr = {22'h10_0000};  //ba0 = 1, sys_ad[20] = 1
      end
   endtask // EMRS   

   task MRS;
      input [2:0] burst_length;
      input 	     burst_type;
      input [2:0] cas_latency;
      begin
	 sys_cmd = `sys_load_mr;
	 sys_addr = {12'h000,1'b0,cas_latency,burst_type,burst_length};
      end
   endtask // MRS

   task auto_refresh;
      begin
	 sys_cmd = `sys_refresh;
	 #(`PERIOD);
	 sys_cmd = `sys_nop;
      end
   endtask // auto_refresh

   task read_autoprecharge;
      input [1:0]  ba;
      input [`ROW_ADDR_MSB:0] row_addr;
      input [`COL_ADDR_MSB:0] col_addr;
      begin
	 sys_cmd = `sys_read;
	 sys_addr = {ba,row_addr,col_addr};
	 #(`PERIOD);
	 sys_cmd = `sys_nop;
	 #(9*`PERIOD);
      end
   endtask // read_autoprecharge

   task write_autoprecharge; //for burst length = 4
      input [1:0]  ba;
      input [`ROW_ADDR_MSB:0] row_addr;
      input [`COL_ADDR_MSB:0] col_addr;
      input [`SYS_DATA_MSB:0] write_data1,write_data2,write_data3,write_data4;
      begin	 
	 sys_cmd = `sys_write;
	 sys_addr = {ba,row_addr,col_addr};
	 sys_data_i = write_data1;
	 #(`PERIOD);  
	 sys_data_i = write_data2;
	 #(`PERIOD);  
	 sys_data_i = write_data3;
	 #(`PERIOD);  
	 sys_data_i = write_data4;
      end
   endtask // write_autoprecharge
   task burst_stop;
      sys_cmd = `sys_burst_stop;
   endtask // burst_stop

   task precharge_all_banks;
      begin      
	 sys_cmd = `sys_precharge;
      end
   endtask // precharge

   task nop;
      sys_cmd = `sys_nop;
   endtask // nop
   

   //for virsim
   initial $vcdpluson(test);
   

endmodule
 