/******************************************************************************
 *
 *    File Name:  tb_post_route.v
 *      Version:  2.5
 *         Date:  June 05, 2000
 * 
 *  Description:  Backannotated test bench.
 * 
 *       Author:  Jennnifer Tran
 *      Company:  Xilinx
 *
 *                Copyright (c) 1999 Xilinx, Inc.
 *                All rights reserved
 *
 *    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
 * 
 *   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
 *                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
 *                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 *                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
 *
 ******************************************************************************/

`include "../src/define.v"
`include "../ram_model/mt46v4m16-133.v"

`timescale 1ns / 1ps 
`define PERIOD       7.5
`define HALF_PERIOD  3.75
`define DELAY_TIME   0.2

`define sys_load_mr            5'b00001
`define sys_read               5'b00010
`define sys_write              5'b00100
`define sys_precharge          5'b01000
`define sys_refresh            5'b10000
`define sys_nop                5'b00000

module test();   
   reg [`SYS_ADDR_MSB:0] sys_addr;
   reg [6:2] 		 sys_cmd;
   reg 			 sys_reset_n;
   reg 			 sys_clk;
   reg [`SYS_DATA_MSB:0] sys_data_i;
   
   wire [`DDR_ADDR_MSB:0] ddr_ad;
   wire [`DDR_DATA_MSB:0] ddr_dq; 
   wire [1:0] 		  ddr_dm;
   wire [1:0] 		  ddr_ba;
   wire [1:0] 		  ddr_dqs;
   wire 		  ddr_rasb;
   wire 		  ddr_casb;
   wire 		  ddr_web;

   wire [`SYS_DATA_MSB:0] sys_data_o;
   
   reg 			  GSR;
   
`include "string_decode_ba.v"

   Synopsys_edif utt(.sys_reset_n(sys_reset_n), .sys_ref_ack(sys_ref_ack), 
		     .sys_data_valid(sys_data_valid), .ddr_web(ddr_web), 
		     .ddr_rasb(ddr_rasb), .ddr_csb(ddr_csb), 
		     .ddr_clkb(ddr_clkb), .ddr_clk(ddr_clk), 
		     .ddr_cke(ddr_cke), .ddr_casb(ddr_casb), 
		     .u_clk_fb(sys_clk_fb), .u_clk(sys_clk), 
		     .sys_data_o(sys_data_o), .sys_data_i(sys_data_i), 
		     .sys_cmd(sys_cmd), .sys_addr(sys_addr), 
		     .ddr_dq(ddr_dq), .ddr_dqs(ddr_dqs), 
		     .ddr_dm(ddr_dm), .ddr_ba(ddr_ba), 
		     .ddr_ad(ddr_ad));

   assign  sys_clk_fb = ddr_clk;  //feed DDR clock back to FPGA 
   
   mt46v4m16 u_ddr (.Clk(ddr_clk), .Clk_n(ddr_clkb), .Cs_n(ddr_csb), 
	      .Cke(ddr_cke), .Ba(ddr_ba), .Addr(ddr_ad), 
	      .Ras_n(ddr_rasb), .Cas_n(ddr_casb), .We_n(ddr_web), 
	      .Dm(ddr_dm), .Dq(ddr_dq), .Dqs(ddr_dqs[0]));
  
   always #(`HALF_PERIOD) sys_clk <= ~sys_clk;

   initial begin
      sys_cmd = `sys_nop;
      sys_addr = 22'h00000;
      sys_data_i = 32'h0000000;
      sys_clk = 0;
      sys_reset_n = 1;
     
      
      #(32*`PERIOD);   //wait for DLL in FPGA to lock
      #(`HALF_PERIOD); //delay inputs to avoid race condition
      #(`DELAY_TIME); precharge_all_banks;
      #(5*`PERIOD);     nop;
      #(`PERIOD);     EMRS;
      
      run_test(3'b011,3'b010); // burst = 8, cas latency = 2;
      run_test(3'b011,3'b110); // burst = 8, cas latency = 2.5;
		     
      run_test(3'b010,3'b010); // burst = 4, cas latency = 2;
      run_test(3'b010,3'b110); // burst = 4, cas latency = 2.5;
		     
      run_test(3'b001,3'b010); // burst = 2, cas latency = 2;
      run_test(3'b001,3'b110); // burst = 2, cas latency = 2.5;

      $finish;
   end // initial begin
   
   task run_test;
      input [2:0] burst_length;
      input [2:0] cas_latency;
      begin
	 #(`PERIOD);     nop;
	 #(`PERIOD);     MRS(burst_length,0,cas_latency); 
	 #(`PERIOD);     nop;
	 #(`PERIOD);     precharge_all_banks;
	 #(`PERIOD);     nop;
	 #(2*`PERIOD);   auto_refresh;
	 #(`PERIOD);     nop;
	 #(8*`PERIOD);   auto_refresh;
	 #(`PERIOD);     nop;
	 //write_autoprecharge (ba,row_addr,col_addr,write_data1,write_data2,write_data3,write_data4)
	 #(8*`PERIOD);   write_autoprecharge(2'b00, 12'h200, 8'h10, 32'h10102020, 32'h03030404, 32'h50506060, 32'h07070808);
	 #(`PERIOD);     nop;
	 #(10*`PERIOD);  read_autoprecharge(2'b00, 12'h200, 8'h10);
	 #(9*`PERIOD);   nop;
	 #(10*`PERIOD);  write_autoprecharge(2'b01, 12'h200, 8'h20, 32'h09876543, 32'h98765432, 32'h87654321, 32'h76543210);
	 #(`PERIOD);     nop;
	 #(10*`PERIOD);  read_autoprecharge(2'b00, 12'h200, 8'h10);
	 #(9*`PERIOD);   nop;
	 #(10*`PERIOD);  read_autoprecharge(2'b01, 12'h200, 8'h20);
	 #(9*`PERIOD);   nop;
	 #(10*`PERIOD);
      end
   endtask // run_test   

   task EMRS;
      begin
	 sys_cmd = `sys_load_mr;
	 sys_addr = {22'h10_0000};  //ba0 = 1, sys_ad[20] = 1
      end
   endtask // EMRS   

   task MRS;
      input [2:0] burst_length;
      input 	     burst_type;
      input [2:0] cas_latency;
      begin
	 sys_cmd = `sys_load_mr;
	 sys_addr = {12'h000,1'b0,cas_latency,burst_type,burst_length};
      end
   endtask // MRS

   task auto_refresh;
      begin
	 sys_cmd = `sys_refresh;
	 #(`PERIOD);
	 sys_cmd = `sys_nop;
      end
   endtask // auto_refresh

   task read_autoprecharge;
      input [1:0]  ba;
      input [`ROW_ADDR_MSB:0] row_addr;
      input [`COL_ADDR_MSB:0] col_addr;
      begin
	 sys_cmd = `sys_read;
	 sys_addr = {ba,row_addr,col_addr};
	 #(`PERIOD);
	 sys_cmd = `sys_nop;
	 #(9*`PERIOD);
      end
   endtask // read_autoprecharge

   task write_autoprecharge; //for burst length = 4
      input [1:0]  ba;
      input [`ROW_ADDR_MSB:0] row_addr;
      input [`COL_ADDR_MSB:0] col_addr;
      input [`SYS_DATA_MSB:0] write_data1,write_data2,write_data3,write_data4;
      begin	 
	 sys_cmd = `sys_write;
	 sys_addr = {ba,row_addr,col_addr};
	 sys_data_i = write_data1;
	 #(`PERIOD);  
	 sys_data_i = write_data2;
	 #(`PERIOD);  
	 sys_data_i = write_data3;
	 #(`PERIOD);  
	 sys_data_i = write_data4;
      end
   endtask // write_autoprecharge
   task burst_stop;
      sys_cmd = `sys_burst_stop;
   endtask // burst_stop

   task precharge_all_banks;
      begin      
	 sys_cmd = `sys_precharge;
      end
   endtask // precharge

   task nop;
      sys_cmd = `sys_nop;
   endtask // nop
   

   //for virsim
   initial $vcdpluson(test);
   

endmodule
 