--*****************************************************************************
--
--    File Name:  brst_cntr.vhd
--      Version:  2.5
--         Date:  June 05, 2000
-- 
--  Description:  generate burst_end 
--                For burst of 2 & 4, controller never goes to 
--                   READ_DATA & WRITE_DATA states
--                For burst of 8: controller stays in 
--                   READ_DATA or WRITE_DAT untill burst_end is true
--                burst_end follows ld_burst by 2 clk cycle
--
--       Author:  Jennnifer Tran
--      Company:  Xilinx
--
--                Copyright (c) 1999 Xilinx, Inc.
--                All rights reserved
--
--    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
-- 
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--*****************************************************************************/
library IEEE;
use IEEE.std_logic_1164.all;
library work;
use WORK.ddr_controller_comp.all;
-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity brst_cntr is
  port(
    --Inputs
    clk       : in std_logic;
    ld_brst   : in std_logic;
    
    --Output
    brst_end  : out std_logic);
end brst_cntr;

architecture rtl of brst_cntr is
  signal b_1         : std_logic;
  signal ld_brst_n   : std_logic;

begin
  ld_brst_n <= not(ld_brst);
  
  process (clk)
  begin
    if (clk 'event and clk = '1') then
      b_1 <= ld_brst_n;
      brst_end <= b_1;
    end if;
  end process;
    
end rtl;
