--*****************************************************************************
--
--    File Name:  cslt_cntr.vhd
--      Version:  2.5
--         Date:  June 05, 2000
-- 
--  Description:  counts from cslt_max down to 0.  When counter = 0, cslt_end goes high
--
--       Author:  Jennnifer Tran
--      Company:  Xilinx
--
--                Copyright (c) 1999 Xilinx, Inc.
--                All rights reserved
--
--    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
-- 
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--*****************************************************************************/

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
library work;
use WORK.ddr_controller_comp.all;
-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity cslt_cntr is
  port(
      --Inputs
      cslt_max     : in std_logic_vector(1 downto 0);
      clk          : in std_logic;
      clk2x        : in std_logic;
      ld_cslt      : in std_logic;
      cas_lat_half : in std_logic;
      --Output
      cslt_end     : out std_logic);
end cslt_cntr;

architecture rtl of cslt_cntr is
  signal count        : std_logic_vector(1 downto 0);
  signal cslt_end1    : std_logic;
  signal cslt_end1_5  : std_logic;

begin
  
  cslt_end1 <= not (count(0) or count(1));

  process (clk) begin
    if (clk 'event and clk = '1') then
      if (cas_lat_half = '1') then cslt_end <= cslt_end1;
      else                   cslt_end <= cslt_end1_5;
      end if;
    end if;
  end process;
  
  process (clk) begin
    if (clk 'event and clk = '1') then
      if (ld_cslt = '1') then count <= cslt_max;
      else              count <= count - 1;
      end if;
    end if;     
  end process;

  process (clk2x) begin
    if (clk2x 'event and clk2x = '1') then
      cslt_end1_5 <= cslt_end1;
    end if;
  end process;

end rtl;
