/******************************************************************************
 *
 *    File Name:  ddr_ctlr.v
 *      Version:  2.5
 *         Date:  June 05, 2000
 * 
 *  Description:  controller for DDR SDRAM  
 * 
 *       Author:  Jennnifer Tran
 *      Company:  Xilinx
 *
 *                Copyright (c) 1999 Xilinx, Inc.
 *                All rights reserved
 *
 *    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
 * 
 *   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
 *                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
 *                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 *                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
 *
 ******************************************************************************/
`timescale 100ps / 10ps
`include "../src/define.v"

module ddr_ctlr (/*AUTOARG*/
   // Outputs
   ddr_ad, ddr_dm, ddr_ba, ddr_rasb, ddr_casb, ddr_web, ddr_clk, 
   ddr_clkb, ddr_dqs, ddr_csb, ddr_cke, u_data_o, u_ref_ack, 
   u_data_valid, fpga_clk, 
   // Inouts
   ddr_dq, 
   // Inputs
   u_addr, u_data_i, u_cmd, u_reset_n, u_clk, u_clk_fb
   );

   output [`DDR_ADDR_MSB:0] ddr_ad;
   inout [`DDR_DATA_MSB:0]  ddr_dq;   
   output [1:0] 	    ddr_dm;
   output [1:0] 	    ddr_ba; 
   output 		    ddr_rasb, ddr_casb, ddr_web;
   output 		    ddr_clk, ddr_clkb;
   output [1:0] 	    ddr_dqs;
   output 		    ddr_csb, ddr_cke;
   
   input [`U_ADDR_MSB:0]  u_addr;
   input [`U_DATA_MSB:0]  u_data_i;
   output [`U_DATA_MSB:0] u_data_o;
   input [7:1] 		  u_cmd;
   input 		  u_reset_n;
   input 		  u_clk;
   output 		  u_ref_ack; 		    
   input 		  u_clk_fb;
   output 		  u_data_valid;
   
   output 		  fpga_clk;

   wire [1:0] 		  cas_lat_half;
   wire [1:0] 		  ddr_burst_length;
   wire [2:0] 		  ddr_cas_latency;
   wire [1:0] 		  cas_lat_max;
   wire 		  dlls_locked;		    
   wire 		  reset = ~(dlls_locked & u_reset_n);
   wire [7:0] 		  ddr_write_en;
   wire [`ENABLE_MSB:0]   ddr_read_en; 		    
   
   //fanout reset to reduce net delay
   reg 			  ctlr_reset1, ctlr_reset2, ctlr_reset3;
   reg 			  addr_reset1, addr_reset2;
   always @(posedge fpga_clk) begin
      ctlr_reset1 <= reset;
      ctlr_reset2 <= reset;
      ctlr_reset3 <= reset;
      addr_reset1 <= reset;
      addr_reset2 <= reset; 
   end
   
   /*********************************************************\
    *       Instantiation of sub modules.                   *
    *                                                       *
   \********************************************************/   
   clk_dlls clk_dlls (.ddr_clk_out(ddr_clk), .ddr_clk180_out(ddr_clkb), 
		      .fpga_clk2x_out(fpga_clk2x), .fpga_clk_out(fpga_clk), 
		      .dlls_locked_out(dlls_locked), .sys_clk_in(u_clk), 
		      .sys_clk_fb_in(u_clk_fb), .fpga_lac_clk_out(fpga_lac_clk), 
		      .reset_in(1'b0), .fpga_lac_clk90_out(fpga_lac_clk90));
   
   data_path data_path (.u_data_o(u_data_o), .ddr_dqs(ddr_dqs), 
			.u_data_valid(u_data_valid), .burst_8(burst_8),
			.ddr_dq(ddr_dq), .u_data_i(u_data_i), 
			.ddr_dqs_t(ddr_dqs_t), .u_data_valid_en(u_data_valid_en),
			.ddr_write_en(ddr_write_en), .ddr_read_en(ddr_read_en), 
			.clk2x(fpga_clk2x), .clk(fpga_clk),
			.lac_clk_in(fpga_lac_clk), .cas_lat_half(cas_lat_half));

   addr_latch addr_latch(.burst_2(burst_2), .burst_8(burst_8), 
			 .cas_lat_max(cas_lat_max), .cas_lat_half(cas_lat_half),
			 .ddr_ad(ddr_ad), .ddr_ba(ddr_ba), .u_addr(u_addr), 
			 .row_addr(row_addr), .mrs_addr(mrs_addr),.clk(fpga_clk), 
			 .reset2(addr_reset2), .reset1(addr_reset1));

   controller controller (.ddr_rasb(ddr_rasb), .ddr_casb(ddr_casb), 
			  .ddr_web(ddr_web), .ld_burst(ld_burst), 
			  .ld_rcd(ld_rcd), .ld_cas_lat(ld_cas_lat), 
			  .u_ref_ack(u_ref_ack), .ddr_read_en(ddr_read_en), 
			  .ddr_dqs_t(ddr_dqs_t), .u_data_valid_en(u_data_valid_en),
			  .ddr_write_en(ddr_write_en),
			  .mrs_addr(mrs_addr), .row_addr(row_addr), 
			  .burst_2(burst_2), .burst_8(burst_8),
			  .u_cmd(u_cmd), .cntlr_reset1(ctlr_reset1), 
			  .cntlr_reset2(ctlr_reset2), .cntlr_reset3(ctlr_reset3),
			  .clk2x(fpga_clk2x), .lac_clk_in(fpga_lac_clk90), 
			  .clk(fpga_clk), .burst_end(burst_end), 
			  .cas_lat_end(cas_lat_end), .rcd_end(rcd_end)
			   );
    
   brst_cntr brst_cntr   (.brst_end(burst_end),
			  .clk(fpga_clk), .ld_brst(ld_burst));

   cslt_cntr cslt_cntr   (.cslt_end(cas_lat_end), .cslt_max(cas_lat_max),
			  .cas_lat_half(cas_lat_half[1]), .clk(fpga_clk), 
			  .clk2x(fpga_clk2x), .ld_cslt(ld_cas_lat));

   rcd_cntr rcd_cntr     (.rcd_end(rcd_end),
			  .clk(fpga_clk), .ld_rcd(ld_rcd));

   //interface to DDR signals
   //all inputs/outputs use SSTL2_II
   OBUF_SSTL2_I O_ddr_dqm0 (.O(ddr_dm[0]), .I(1'b0));
   OBUF_SSTL2_I O_ddr_dqm1 (.O(ddr_dm[1]), .I(1'b0));
   OBUF_SSTL2_I O_ddr_cke  (.O(ddr_cke), .I(1'b1));
   OBUF_SSTL2_I O_ddr_csb  (.O(ddr_csb), .I(1'b0));  
   
endmodule




