/******************************************************************************
 *
 *    File Name:  user_int.v
 *      Version:  2.5
 *         Date:  June 05, 2000
 * 
 *  Description:  dummy user_int module.
 *                In order to place and route the ddr_ctrl module, we need to
 *                connect all user interface signals.  The user_int module simply
 *                connects all user signals to IOs.  
 * 
 *                The user should replace user_int module with their own interface.
 * 
 * 
 *       Author:  Jennnifer Tran
 *      Company:  Xilinx
 *
 *                Copyright (c) 1999 Xilinx, Inc.
 *                All rights reserved
 *
 *    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
 * 
 *   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
 *                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
 *                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 *                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
 *
 ******************************************************************************/

`timescale 100ps / 10ps
`include "../src/define.v"

module user_int (/*AUTOARG*/
   // Outputs
   sys_data_o, sys_ref_ack, sys_data_valid, u_addr, u_data_i, u_cmd, 
   u_reset_n, u_clk, u_clk_fb, 
   // Inputs
   sys_addr, sys_data_i, sys_cmd, sys_reset_n, sys_clk, sys_clk_fb, 
   u_data_o, u_ref_ack, u_data_valid, fpga_clk
   );

   input [`SYS_ADDR_MSB:0]  sys_addr;
   input [`SYS_DATA_MSB:0]  sys_data_i;
   output [`SYS_DATA_MSB:0] sys_data_o;
   input [7:1] 		    sys_cmd;
   input 		    sys_reset_n;
   input 		    sys_clk;
   output 		    sys_ref_ack; 		    
   input 		    sys_clk_fb;
   output 		    sys_data_valid;
   
   output [`U_ADDR_MSB:0] u_addr;
   output [`U_DATA_MSB:0] u_data_i;
   input [`U_DATA_MSB:0]  u_data_o;
   output [7:1] 	  u_cmd;
   output 		  u_reset_n;
   output 		  u_clk;
   input 		  u_ref_ack; 		    
   output 		  u_clk_fb;
   input 		  u_data_valid;
   input 		  fpga_clk;

   wire [`DDR_ADDR_MSB:0] ddr_ad;
   wire [`DDR_DATA_MSB:0] ddr_dq;
   wire [1:0] 		  ddr_dm;
   wire [1:0] 		  ddr_ba; 
   wire [1:0] 		  ddr_dqs;
   
   wire [`SYS_ADDR_MSB:0] sys_addr_pad;
   wire [`SYS_DATA_MSB:0] sys_data_i_pad;
   reg [`SYS_DATA_MSB:0]  sys_data_o_pad;
   wire [7:1] 		  sys_cmd_pad;
   wire 		  sys_reset_n_pad;
   reg 			  sys_data_valid_pad;
   
   //register all system inputs to in IOBs, except clk signals
   reg [`U_ADDR_MSB:0] u_addr;
   reg [`U_DATA_MSB:0] u_data_i;
   reg [7:1] 	       u_cmd;
   reg 		       u_reset_n;
   wire [`SYS_DATA_MSB:0] sys_data_o;
   wire 		  sys_data_valid;
   
   always @(posedge fpga_clk) begin      
      u_addr <= sys_addr_pad;
      u_data_i <= sys_data_i_pad;
      u_cmd <= sys_cmd_pad;
      u_reset_n <= sys_reset_n_pad;
      sys_data_o_pad <= u_data_o;
      sys_data_valid_pad <= u_data_valid;
   end
   wire 		   u_clk = sys_clk;
   wire 		   sys_ref_ack_pad = u_ref_ack;
   wire 		   u_clk_fb = sys_clk_fb;


   //instantiate IO buffers
   IBUF_SSTL2_I U_a0 (.I(sys_addr[0]), .O(sys_addr_pad[0]));
   IBUF_SSTL2_I U_a1 (.I(sys_addr[1]), .O(sys_addr_pad[1]));
   IBUF_SSTL2_I U_a2 (.I(sys_addr[2]), .O(sys_addr_pad[2]));
   IBUF_SSTL2_I U_a3 (.I(sys_addr[3]), .O(sys_addr_pad[3]));
   IBUF_SSTL2_I U_a4 (.I(sys_addr[4]), .O(sys_addr_pad[4]));
   IBUF_SSTL2_I U_a5 (.I(sys_addr[5]), .O(sys_addr_pad[5]));
   IBUF_SSTL2_I U_a6 (.I(sys_addr[6]), .O(sys_addr_pad[6]));
   IBUF_SSTL2_I U_a7 (.I(sys_addr[7]), .O(sys_addr_pad[7]));
   IBUF_SSTL2_I U_a8 (.I(sys_addr[8]), .O(sys_addr_pad[8]));
   IBUF_SSTL2_I U_a9 (.I(sys_addr[9]), .O(sys_addr_pad[9]));
   IBUF_SSTL2_I U_a10 (.I(sys_addr[10]), .O(sys_addr_pad[10]));
   IBUF_SSTL2_I U_a11 (.I(sys_addr[11]), .O(sys_addr_pad[11]));
   IBUF_SSTL2_I U_a12 (.I(sys_addr[12]), .O(sys_addr_pad[12]));
   IBUF_SSTL2_I U_a13 (.I(sys_addr[13]), .O(sys_addr_pad[13]));
   IBUF_SSTL2_I U_a14 (.I(sys_addr[14]), .O(sys_addr_pad[14]));
   IBUF_SSTL2_I U_a15 (.I(sys_addr[15]), .O(sys_addr_pad[15]));
   IBUF_SSTL2_I U_a16 (.I(sys_addr[16]), .O(sys_addr_pad[16]));
   IBUF_SSTL2_I U_a17 (.I(sys_addr[17]), .O(sys_addr_pad[17]));
   IBUF_SSTL2_I U_a18 (.I(sys_addr[18]), .O(sys_addr_pad[18]));
   IBUF_SSTL2_I U_a19 (.I(sys_addr[19]), .O(sys_addr_pad[19]));
   IBUF_SSTL2_I U_a20 (.I(sys_addr[20]), .O(sys_addr_pad[20]));
   IBUF_SSTL2_I U_a21 (.I(sys_addr[21]), .O(sys_addr_pad[21]));


   IBUF_SSTL2_I U_di0 (.I(sys_data_i[0]), .O(sys_data_i_pad[0]));
   IBUF_SSTL2_I U_di1 (.I(sys_data_i[1]), .O(sys_data_i_pad[1]));
   IBUF_SSTL2_I U_di2 (.I(sys_data_i[2]), .O(sys_data_i_pad[2]));
   IBUF_SSTL2_I U_di3 (.I(sys_data_i[3]), .O(sys_data_i_pad[3]));
   IBUF_SSTL2_I U_di4 (.I(sys_data_i[4]), .O(sys_data_i_pad[4]));
   IBUF_SSTL2_I U_di5 (.I(sys_data_i[5]), .O(sys_data_i_pad[5]));
   IBUF_SSTL2_I U_di6 (.I(sys_data_i[6]), .O(sys_data_i_pad[6]));
   IBUF_SSTL2_I U_di7 (.I(sys_data_i[7]), .O(sys_data_i_pad[7]));
   IBUF_SSTL2_I U_di8 (.I(sys_data_i[8]), .O(sys_data_i_pad[8]));
   IBUF_SSTL2_I U_di9 (.I(sys_data_i[9]), .O(sys_data_i_pad[9]));
   IBUF_SSTL2_I U_di10 (.I(sys_data_i[10]), .O(sys_data_i_pad[10]));
   IBUF_SSTL2_I U_di11 (.I(sys_data_i[11]), .O(sys_data_i_pad[11]));
   IBUF_SSTL2_I U_di12 (.I(sys_data_i[12]), .O(sys_data_i_pad[12]));
   IBUF_SSTL2_I U_di13 (.I(sys_data_i[13]), .O(sys_data_i_pad[13]));
   IBUF_SSTL2_I U_di14 (.I(sys_data_i[14]), .O(sys_data_i_pad[14]));
   IBUF_SSTL2_I U_di15 (.I(sys_data_i[15]), .O(sys_data_i_pad[15]));
   IBUF_SSTL2_I U_di16 (.I(sys_data_i[16]), .O(sys_data_i_pad[16]));
   IBUF_SSTL2_I U_di17 (.I(sys_data_i[17]), .O(sys_data_i_pad[17]));
   IBUF_SSTL2_I U_di18 (.I(sys_data_i[18]), .O(sys_data_i_pad[18]));
   IBUF_SSTL2_I U_di19 (.I(sys_data_i[19]), .O(sys_data_i_pad[19]));
   IBUF_SSTL2_I U_di20 (.I(sys_data_i[20]), .O(sys_data_i_pad[20]));
   IBUF_SSTL2_I U_di21 (.I(sys_data_i[21]), .O(sys_data_i_pad[21]));
   IBUF_SSTL2_I U_di22 (.I(sys_data_i[22]), .O(sys_data_i_pad[22]));
   IBUF_SSTL2_I U_di23 (.I(sys_data_i[23]), .O(sys_data_i_pad[23]));
   IBUF_SSTL2_I U_di24 (.I(sys_data_i[24]), .O(sys_data_i_pad[24]));
   IBUF_SSTL2_I U_di25 (.I(sys_data_i[25]), .O(sys_data_i_pad[25]));
   IBUF_SSTL2_I U_di26 (.I(sys_data_i[26]), .O(sys_data_i_pad[26]));
   IBUF_SSTL2_I U_di27 (.I(sys_data_i[27]), .O(sys_data_i_pad[27]));
   IBUF_SSTL2_I U_di28 (.I(sys_data_i[28]), .O(sys_data_i_pad[28]));
   IBUF_SSTL2_I U_di29 (.I(sys_data_i[29]), .O(sys_data_i_pad[29]));
   IBUF_SSTL2_I U_di30 (.I(sys_data_i[30]), .O(sys_data_i_pad[30]));
   IBUF_SSTL2_I U_di31 (.I(sys_data_i[31]), .O(sys_data_i_pad[31]));


   IBUF_SSTL2_I U_cmd1 (.I(sys_cmd[1]), .O(sys_cmd_pad[1]));
   IBUF_SSTL2_I U_cmd2 (.I(sys_cmd[2]), .O(sys_cmd_pad[2]));
   IBUF_SSTL2_I U_cmd3 (.I(sys_cmd[3]), .O(sys_cmd_pad[3]));
   IBUF_SSTL2_I U_cmd4 (.I(sys_cmd[4]), .O(sys_cmd_pad[4]));
   IBUF_SSTL2_I U_cmd5 (.I(sys_cmd[5]), .O(sys_cmd_pad[5]));
   IBUF_SSTL2_I U_cmd6 (.I(sys_cmd[6]), .O(sys_cmd_pad[6]));
   IBUF_SSTL2_I U_cmd7 (.I(sys_cmd[7]), .O(sys_cmd_pad[7]));

   IBUF_SSTL2_I U_reset (.I(sys_reset_n), .O(sys_reset_n_pad));

   OBUF_SSTL2_I U_data_valid (.O(sys_data_valid),  .I(sys_data_valid_pad));
   OBUF_SSTL2_I U_ref_ack (.O(sys_ref_ack),  .I(sys_ref_ack_pad));
 
   OBUF_SSTL2_I U_do0  (.O(sys_data_o[0]),  .I(sys_data_o_pad[0]));
   OBUF_SSTL2_I U_do1  (.O(sys_data_o[1]),  .I(sys_data_o_pad[1]));
   OBUF_SSTL2_I U_do2  (.O(sys_data_o[2]),  .I(sys_data_o_pad[2]));
   OBUF_SSTL2_I U_do3  (.O(sys_data_o[3]),  .I(sys_data_o_pad[3]));
   OBUF_SSTL2_I U_do4  (.O(sys_data_o[4]),  .I(sys_data_o_pad[4]));
   OBUF_SSTL2_I U_do5  (.O(sys_data_o[5]),  .I(sys_data_o_pad[5]));
   OBUF_SSTL2_I U_do6  (.O(sys_data_o[6]),  .I(sys_data_o_pad[6]));
   OBUF_SSTL2_I U_do7  (.O(sys_data_o[7]),  .I(sys_data_o_pad[7]));
   OBUF_SSTL2_I U_do8  (.O(sys_data_o[8]),  .I(sys_data_o_pad[8]));
   OBUF_SSTL2_I U_do9  (.O(sys_data_o[9]),  .I(sys_data_o_pad[9]));
   OBUF_SSTL2_I U_do10 (.O(sys_data_o[10]), .I(sys_data_o_pad[10]));
   OBUF_SSTL2_I U_do11 (.O(sys_data_o[11]), .I(sys_data_o_pad[11]));
   OBUF_SSTL2_I U_do12 (.O(sys_data_o[12]), .I(sys_data_o_pad[12]));
   OBUF_SSTL2_I U_do13 (.O(sys_data_o[13]), .I(sys_data_o_pad[13]));
   OBUF_SSTL2_I U_do14 (.O(sys_data_o[14]), .I(sys_data_o_pad[14]));
   OBUF_SSTL2_I U_do15 (.O(sys_data_o[15]), .I(sys_data_o_pad[15]));
   OBUF_SSTL2_I U_do16 (.O(sys_data_o[16]), .I(sys_data_o_pad[16]));
   OBUF_SSTL2_I U_do17 (.O(sys_data_o[17]), .I(sys_data_o_pad[17]));
   OBUF_SSTL2_I U_do18 (.O(sys_data_o[18]), .I(sys_data_o_pad[18]));
   OBUF_SSTL2_I U_do19 (.O(sys_data_o[19]), .I(sys_data_o_pad[19]));
   OBUF_SSTL2_I U_do20 (.O(sys_data_o[20]), .I(sys_data_o_pad[20]));
   OBUF_SSTL2_I U_do21 (.O(sys_data_o[21]), .I(sys_data_o_pad[21]));
   OBUF_SSTL2_I U_do22 (.O(sys_data_o[22]), .I(sys_data_o_pad[22]));
   OBUF_SSTL2_I U_do23 (.O(sys_data_o[23]), .I(sys_data_o_pad[23]));
   OBUF_SSTL2_I U_do24 (.O(sys_data_o[24]), .I(sys_data_o_pad[24]));
   OBUF_SSTL2_I U_do25 (.O(sys_data_o[25]), .I(sys_data_o_pad[25]));
   OBUF_SSTL2_I U_do26 (.O(sys_data_o[26]), .I(sys_data_o_pad[26]));
   OBUF_SSTL2_I U_do27 (.O(sys_data_o[27]), .I(sys_data_o_pad[27]));
   OBUF_SSTL2_I U_do28 (.O(sys_data_o[28]), .I(sys_data_o_pad[28]));
   OBUF_SSTL2_I U_do29 (.O(sys_data_o[29]), .I(sys_data_o_pad[29]));
   OBUF_SSTL2_I U_do30 (.O(sys_data_o[30]), .I(sys_data_o_pad[30]));
   OBUF_SSTL2_I U_do31 (.O(sys_data_o[31]), .I(sys_data_o_pad[31]));
		   
endmodule // top

